/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_LANECONTROL_H
#define DPC_IMPL_TYPE_LANECONTROL_H

#include <cfloat>
#include <cmath>
#include "impl_type_float.h"

namespace dpc {
struct LaneControl {
    ::Float start_range{ 0.0F };
    ::Float end_range{ 0.0F };
    ::Float value{ 0.0F };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(start_range);
        fun(end_range);
        fun(value);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(start_range);
        fun(end_range);
        fun(value);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("start_range", start_range);
        fun("end_range", end_range);
        fun("value", value);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("start_range", start_range);
        fun("end_range", end_range);
        fun("value", value);
    }

    friend bool operator==(const ::dpc::LaneControl& lhs, const ::dpc::LaneControl& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.start_range - rhs.start_range)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.end_range - rhs.end_range)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.value - rhs.value)) < DBL_EPSILON);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_LANECONTROL_H
